
beql = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "AceConsole-2.0","FuBarPlugin-2.0")
local L = AceLibrary("AceLocale-2.2"):new("beql")
--SetLocale("ruRU") -- Remove the -- for russian
local tablet = AceLibrary("Tablet-2.0")
local beqldewdrop = AceLibrary("Dewdrop-2.0")
beqlwaterfall = AceLibrary("Waterfall-1.0")
local beql, self = beql, beql
local VERSION = "0.81"
beql.revision = VERSION
beql.versionstring = VERSION

-- Debug functions

function beql:Print(text)
    DEFAULT_CHAT_FRAME:AddMessage(tostring(text))
end

function beql:Nyelv(l)
	if not l or l == "" then
		return
	else
		L:SetLocale(l)
		DEFAULT_CHAT_FRAME:AddMessage("Locale set to:"..l.." (reload interface)")
	end
end

-- Addon functions

function beql:OnInitialize()
	beql:RegisterDB("beqlDB","beqlDBPC","char")
	beql:SetProfile("char")    
	beql:SetupDefaults()
	beql.defaultPosition = "LEFT"
	beql.defaultMinimapPosition = 270
	beql.hasIcon = true
	beql:SetIcon("Interface\\Icons\\INV_Letter_03")
	beql.hideWithoutStandby = true
	beql.cannotDetachTooltip = true
	beql:SetText("bEQL")
	beql:HideText()	
	beql:RegisterChatCommand({"/beql"}, beql.options)
	beql.OnMenuRequest = beql.options	
	beqlwaterfall:Register("bEQL",
	'title',"bEQL",
	'aceOptions',beql.options)
end

function beql:OnEnable()
    
    beql:Compatibility()	
	beql:InitQuestLog()
	if not beql.db.char.disabled.tracker then
		beql:InitTracker()
	end
	
	beql:setmovable(beql.db.char.locked)
	beql:setmovabletracker(beql.db.char.lockedtracker)	
	
	QuestWatch_Update()
	
end

function beql:OnDisable()
	QUESTS_DISPLAYED = 6
	beql:UnhookAll()
	ReloadUI()
end

function beql:Compatibility()
	if IsAddOnLoaded("CT_Core") then
		-- beql:Print(L["bEQL: CT Module found! disabled Quest Level display."])
		-- CT_Core fix
		beql.db.char.disabled.lockedtracker = true
		beql.db.char.lockedtracker = true
		-- leaving there for old users
		beql.db.char.disabled.showlevel = false
	else
		beql.db.char.disabled.showlevel = false
		beql.db.char.disabled.lockedtracker = false
	end
	
	if IsAddOnLoaded("Fubar_QuestsFu") then
		beql:Print(L["bEQL: Fubar_QuestsFu found! disabled bEQL Tracker."])
		beql.db.char.disabled.tracker = true
	else
		beql.db.char.disabled.tracker = false
	end
end

function beql:SetupDefaults()
	
	beql:RegisterDefaults("char", {
		saved = {},
		disabled = {},
		QuestDB = {},
		QuestWatches = {},
		Color = 
		{
			BackgroundColor = {
				r = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g = TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
			},
			BackgroundCornerColor = {
				r = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,
				g = TOOLTIP_DEFAULT_BACKGROUND_COLOR.g,
				b = TOOLTIP_DEFAULT_BACKGROUND_COLOR.r,			
			},
			Zone = {
				r = 1,
				g = 1,
				b = 1,
			},
			HeaderComplete = {
				r = NORMAL_FONT_COLOR.r,
				g = NORMAL_FONT_COLOR.g,
				b = NORMAL_FONT_COLOR.b,
			},
			HeaderEmpty = {
				r = 0.75,
				g = 0.61,
				b = 0,
			},
			ObjectiveEmpty = {
				r = 0.8,
				g = 0.8,
				b = 0.8,
			},
			ObjectiveComplete = {
				r = HIGHLIGHT_FONT_COLOR.r,
				g = HIGHLIGHT_FONT_COLOR.g,
				b = HIGHLIGHT_FONT_COLOR.b,
			}
		},
		
		locked = true,
		alwaysminimize = false,
		alwaysmaximize = false,
		showlevel = true,
		InfoOnQuestCompletion = false,
		autocomplete = false,
		
		lockedtracker = false,
		ShowTrackerHeader = false,
		TrackerFontHeight = 12,
		CustomBackgroundColor = false,
		BackgroundAlpha = false,		
		ShowZonesInTracker = false,
		CustomZoneColor = false,
		CustomObjetiveColor = false,
		RemoveFinished = true,
		MinimizeFinished = false,
		CustomHeaderColor = false,
		HeaderQuestLevelColor = false,
		ShowObjectiveMarkers = true,
		UseTrackerListing = false,
		TrackerList = 0,
		TrackerSymbol = 0,
		SortTrackerItems = false,
		AddNew = true,
		FadeColor = false,
	})		
	

end

function beql:AddQuestStatusToChatFrame(questIndex, questName)
	local text, type, finished
	local chatFrame = DEFAULT_CHAT_FRAME
    local chatType = chatFrame.editBox:GetAttribute("chatType")
    if chatType == "WHISPER" then
		chatType = "SAY"
	end
	local questLogTitleText, level, questTag, groupnr, isHeader, isCollapsed, isComplete = beql.hooks.GetQuestLogTitle(questIndex)
	local numObjectives = GetNumQuestLeaderBoards()

	if numObjectives>0 then
        if questTag then
			SendChatMessage(gsub(questName, " *(.*)", "%1") .. " (" .. questTag .. ")", chatType)
		else
			SendChatMessage(gsub(questName, " *(.*)", "%1"), chatType)
		end
		for i=1, numObjectives, 1 do
			text, type, finished = GetQuestLogLeaderBoard(i,questIndex)
			if not text or strlen(text) == 0 then
				text = type
			end
            if finished then
                text = text.." "..L["(Done)"]
            end
            SendChatMessage(" - "..i..". "..text, chatType)
	    end
	else
	    DEFAULT_CHAT_FRAME:AddMessage(L["No Objectives!"])
	end
end


-- QuestDB !!!
function beql:LookForCompletedQuests(ding)
	if  not beql.db.char.saved.CompletedQuests  then
		beql.db.char.saved.CompletedQuests = {}
	end

	local questID
	local numEntries = GetNumQuestLogEntries()
	local questTitle, isComplete, isHeader

	for i=1, numEntries, 1 do
		questID = i
		questTitle, _, _, _, isHeader, _, isComplete = self.hooks.GetQuestLogTitle(questID)

		if beql.db.char.saved.CompletedQuests[questTitle] then
			beql.db.char.saved.CompletedQuests[questTitle].isOk = true
		else
			beql.db.char.saved.CompletedQuests[questTitle] = {}
			beql.db.char.saved.CompletedQuests[questTitle].isOk = true
		end


		if ( not isHeader ) and isComplete and ( not beql.db.char.saved.CompletedQuests[questTitle].isCompleted )  then
			beql.db.char.saved.CompletedQuests[questTitle].isCompleted = true
			if (ding == nil) or (ding~=false) then
				UIErrorsFrame:AddMessage(questTitle.." "..L["Completed!"], 1.0, 0.8, 0.0, 1.0, UIERRORS_HOLD_TIME)
				PlaySound("GnomeExploration")
			end
		end
	end
	--[[
	--clean up
	for quest in pairs(beql.db.char.saved.CompletedQuests) do
		if beql.db.char.saved.CompletedQuests[quest].isOk then
			beql.db.char.saved.CompletedQuests[quest].isOk = false
		else
			beql.db.char.saved.CompletedQuests[quest] = nil
		end
	end
	]]--
end

function beql:GetQuestHeaderForWatch(questIndex)
	if(beql.db.char.QuestWatches[questIndex]) then
		local numEntries = GetNumQuestLogEntries()
		local questLogHeader, isCollapsed
		local s = beql.db.char.QuestWatches[questIndex]
		local temp=nil
		for w in string.gmatch(s, "[^,]+") do
			if temp == nil then
				temp = w
				break
			end
		end
		for i=1, numEntries, 1 do
			questLogHeader, _, _, _, _, isCollapsed = GetQuestLogTitle(i)
			if questLogHeader == temp then
				return questLogHeader, isCollapsed
			end
		end
		return temp, true
	end
	return "", false
end

function beql:SortWatchedQuests()
	if beql.db.char.ShowZonesInTracker and beql.db.char.SortTrackerItems then
		table.sort(beql.db.char.QuestWatches)
	else
		if beql.db.char.SortTrackerItems then
			table.sort(beql.db.char.QuestWatches, SortCompare);
		end
	end
end

function beql:ManageQuests()
	local tempHeaderList = {}
	local tempQuestList = {}
	local numEntries = GetNumQuestLogEntries()
	local questLogTitleText, level, questTag, isHeader, isCollapsed, isComplete
	local currentHeader = nil
	local temp
	
	local oldTitle
	
	for j=numEntries, 1, -1 do
		questLogTitleText, _, _, _, isHeader, isCollapsed = GetQuestLogTitle(j)
		if (isHeader and isCollapsed) then
			tempHeaderList[j] = 1
			ExpandQuestHeader(j)
		end
	end
	
	local numEntries2 = GetNumQuestLogEntries()
	for j=1, numEntries2, 1 do
		questLogTitleText, level, questTag, _, isHeader, isCollapsed, isComplete = GetQuestLogTitle(j)
		oldTitle = self.hooks.GetQuestLogTitle(j)

		if isHeader then
			currentHeader = questLogTitleText
		else
			temp = currentHeader..","..level..","..questLogTitleText
			table.insert(tempQuestList, temp)

			if beql.TrackerQueue and beql.TrackerQueue == oldTitle then
				table.insert(beql.db.char.QuestWatches, temp)
				beql.TrackerQueue = nil
			end

		end
	end	
	
	for j=1, numEntries, 1 do
		if(tempHeaderList[j] == 1) then
			CollapseQuestHeader(j)
		end
	end
	
	
	local numWatches = table.getn(beql.db.char.QuestWatches)
	local numEntries = table.getn(tempQuestList)
	local found = false
	for i=numWatches, 1, -1 do
		found = false
		for j=0, numEntries, 1 do
			if beql.db.char.QuestWatches[i] == tempQuestList[j] then
				found = true
				break
			end
		end
		if not found then
			table.remove(beql.db.char.QuestWatches, i)
		end
	end	
	
	beql:SortWatchedQuests()
end

function beql:FadeColors(tempColor, tempColor2, done, obj)
	local color = {r=0, g=0, b=0}
	local multiplier

	multiplier = (done / obj)
	color.r = tempColor.r + ((tempColor2.r - tempColor.r)*multiplier)
	color.g = tempColor.g + ((tempColor2.g - tempColor.g)*multiplier)
	color.b = tempColor.b + ((tempColor2.b - tempColor.b)*multiplier)

	return color

end

--- EOF ---